package com.iohao.mmo.equip.action;

import com.iohao.game.action.skeleton.annotation.ActionController;
import com.iohao.game.action.skeleton.annotation.ActionMethod;
import com.iohao.game.action.skeleton.core.flow.FlowContext;
import com.iohao.game.action.skeleton.protocol.wrapper.StringValue;
import com.iohao.mmo.common.config.GameCode;
import com.iohao.mmo.equip.cmd.EquipCmd;
import com.iohao.mmo.equip.entity.Equip;
import com.iohao.mmo.equip.mapper.EquipMapper;
import com.iohao.mmo.equip.proto.CreateEquipMessage;
import com.iohao.mmo.equip.proto.EquipMessage;
import com.iohao.mmo.equip.proto.EquipResetMessage;
import com.iohao.mmo.equip.proto.NewEquipMessage;
import com.iohao.mmo.equip.service.EquipService;
import com.iohao.mmo.equip.service.EquipTemplateService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;

/**
 * @author 唐斌
 * @date 2023-07-30
 * @description: 装备属性类
 */
@Slf4j
@Component
@ActionController(EquipCmd.cmd)
public class EquipAction {
    @Resource
    EquipService equipService;
    @Resource
    EquipTemplateService equipTemplateService;

    /**
     * 获取某人装备列表信息
     *
     * @param flowContext flowContext
     */
    @ActionMethod(EquipCmd.getEquipList)
    public List<EquipMessage> getEquipList(FlowContext flowContext) { //参数值
        long userId = flowContext.getUserId();
        List<Equip> equipList = equipService.listByUser(userId);
        return EquipMapper.ME.convert(equipList);
    }

    /**
     * 获取装备信息
     *
     * @param flowContext flowContext
     * @param stringValue itemTypeId
     */
    @ActionMethod(EquipCmd.getEquip)
    public EquipMessage getEquip(FlowContext flowContext, StringValue stringValue) { //参数值
        Equip equip = equipService.findById(stringValue.value);
        return EquipMapper.ME.convert(equip);
    }

    /**
     * 分配装备属性点
     *
     * @param flowContext flowContext
     */
    @ActionMethod(EquipCmd.allotEquip)
    public void allotEquip(FlowContext flowContext, EquipMessage equipMessage) { //参数值
        equipService.allotEquip(EquipMapper.ME.convert(equipMessage));
    }

    /**
     * 重新随机总属性点
     *
     * @param flowContext       flowContext
     * @param equipResetMessage 重新随机装备总属性点参数对象
     */
    @ActionMethod(EquipCmd.resetEquip)
    public EquipMessage resetEquip(FlowContext flowContext, EquipResetMessage equipResetMessage) { //参数值
        GameCode.objNotFound.assertTrue(StringUtils.isNotBlank(equipResetMessage.id) && StringUtils.isNotBlank(equipResetMessage.excellentRateString));
        Equip equip = equipService.resetEquip((equipResetMessage.id), new BigDecimal(equipResetMessage.excellentRateString));
        return EquipMapper.ME.convert(equip);
    }

    /**
     * 批量删除装备
     *
     * @param flowContext flowContext
     * @param idList 装备id列表
     */
    @ActionMethod(EquipCmd.delEquipBatch)
    public void delEquipBatch(FlowContext flowContext, List<String> idList) {
        equipService.delBatch(idList);
    }

    @ActionMethod(EquipCmd.createEquip)
    public NewEquipMessage internalCreateEquip(CreateEquipMessage createEquipMessage, FlowContext flowContext) {
        // 玩家
        long userId = flowContext.getUserId();

        Equip equip = equipTemplateService.randomEquip(createEquipMessage.itemTypeId, BigDecimal.valueOf(createEquipMessage.excellentRate), userId);

        /*
         * 将新装备信息给到调用方，
         * 装备模块并不关心调用方是谁，只要调用此 action，就生成新装备。
         */

        NewEquipMessage newEquipMessage = new NewEquipMessage();
        newEquipMessage.equipId = equip.getId();
        newEquipMessage.itemTypeId = equip.getItemTypeId();

        return newEquipMessage;
    }
}
